#include <iomacros.h>

#define IO_BASE 0x20

#define PORTA 0x1b
#define PORTB 0x18
#define PORTC 0x15
#define PORTD 0x12

#define DDR(port) ((port)-1)
#define PINS(port) ((port)-2)

#define IO(ioreg) (*((unsigned char *)((ioreg)+IO_BASE)))

//#define SET_LOW(ioreg, bits)  IO(ioreg) = IO(ioreg) & ~(bits)
//#define SET_HIGH(ioreg, bits) IO(ioreg) = IO(ioreg) | (bits)
#define SET_LOW(ioreg, bits)  outp(inp(ioreg) & ~(bits),ioreg);
#define SET_HIGH(ioreg, bits)  outp(inp(ioreg) | (bits),ioreg);

#define SET_INPUT(port, bits) SET_LOW(DDR(port), bits)
#define SET_OUTPUT(port, bits) SET_HIGH(DDR(port), bits)

#define PORT_OUTPUT(port) outp(0xff, DDR(port));
#define PORT_INPUT(port)  outp(0, DDR(port));

#define ISA_ADDR  0x1f    /* PORTC, bits 0-4 */
#define ISA_IORB  0x20    /* PORTC, bit 5, active low */
#define ISA_IOWB  0x40    /* PORTC, bit 6, active low */
#define ISA_RESET 0x80    /* PORTC, bit 7, active high */

#define CLOCK 7372000  /* clock rate (Hz) */

#define delay_500us() ({      \
        asm volatile (        \
                "nop" "\n\t"  \
                "nop" "\n\t"  \
                "nop" "\n\t"  \
                "nop" "\n\t"  \
        );                    \
})

#ifndef delay_500us
static void delay_500us(void)
{
    ;  // AVR rcall/ret => 7 clocks @ 7.372 MHz => ~1000 nsec
}
#endif

static unsigned short delay_25ms(void)
{
    unsigned short count = (int)(0.025*CLOCK/4);

    while(--count) ;   // 4 clocks/iteration
    return count;
}

void ISAout(unsigned char addr, unsigned char val)
{
    unsigned char ctrl = (ISA_IORB | ISA_IOWB) | (addr & ISA_ADDR);

    //void puthexb(unsigned char val);
    //puthexb(val); space();
    outp(ctrl, PORTC);
    PORT_OUTPUT(PORTA);
    outp(val, PORTA);
    outp(ctrl & ~ISA_IOWB, PORTC);
    delay_500us();
    outp(ctrl, PORTC);
}

void ISAoutw(unsigned char addr, unsigned short val)
{
    unsigned char ctrl = (ISA_IORB | ISA_IOWB) | (addr & ISA_ADDR);

    outp(ctrl, PORTC);
    PORT_OUTPUT(PORTA);
    PORT_OUTPUT(PORTB);
    outp(val, PORTA);
    outp(val >> 8, PORTA);
    outp(ctrl & ~ISA_IOWB, PORTC);
    delay_500us();
    outp(ctrl, PORTC);
}

unsigned char ISAin(unsigned char addr)
{
    unsigned char val;
    unsigned char ctrl = (ISA_IORB | ISA_IOWB) | (addr & ISA_ADDR);

    PORT_INPUT(PORTA);
    outp(ctrl, PORTC);
    outp(ctrl & ~ISA_IORB, PORTC);
    delay_500us();
    val = inp(PINS(PORTA));
    outp(ctrl, PORTC);
    return val;
}

unsigned short ISAinw(unsigned char addr)
{
    unsigned short val;
    unsigned char ctrl = (ISA_IORB | ISA_IOWB) | (addr & ISA_ADDR);

    PORT_INPUT(PORTA);
    PORT_INPUT(PORTB);
    outp(ctrl, PORTC);
    outp(ctrl & ~ISA_IORB, PORTC);
    delay_500us();
    val = inp(PINS(PORTA));
    val |= inp(PINS(PORTB)) << 8;
    outp(ctrl, PORTC);
    return val;
}

void ISAreset(void)
{
    PORT_INPUT(PORTA);
    PORT_INPUT(PORTB);
    PORT_OUTPUT(PORTC);
    outp(ISA_IORB | ISA_IOWB | ISA_RESET, PORTC);
    delay_25ms();
    outp(ISA_IORB | ISA_IOWB, PORTC);
}
