;-------------------------------------------------------------------------
; Copyright (c) 2000 Steven Freyder, David Helland, and Bruce D. Lightner.
; Use subject to licensing restrictions described in "PICOWEB SERVER
; SOFTWARE LICENSE AGREEMENT DATED JULY 14, 1999" (See file LICENSE.txt).
;-------------------------------------------------------------------------

.macro isa_delay        ; delay 500 nsec @ 8 MHz
	nop		; 1 clock  (125 ns / clock @ 8 MHz)
	nop		; 1 clock
	nop		; 1 clock
	nop		; 1 clock
.endm

isa_out:    export        ; r16 = address bits 4:0, i1:i0 = value to output
	push    r17

	ldi     r17,0x60        ; ~ior/~iow inactive
	or      r17,r16
	out     portc,r17       ; assert address lines

	ser     r17             ; 0xff
	out     ddra,r17        ; a/b are outputs
#ifdef BREADBOARD
	out     ddrb,r17
#endif

	out     porta,i0        ; output 16-bits of data
#ifdef BREADBOARD
	out     portb,i1
#endif

	ldi     r17,0x20        ; ~iow active (need >91 nsec delay)
	or      r17,r16
	out     portc,r17       ; do the write pulse/signal

	isa_delay               ; wait > 700 nsec
	ldi     r17,0x60        ; ~ior/~iow inactive
	or      r17,r16         ; keep address lines constant

	out     portc,r17

	pop     r17
	ret

isa_in:     export  ; r16 = address bits 4:0 - yield word result in i1:i0
	push    r17

	clr r17
	out     ddra,r17; a/b are inputs (c is always an output)
#ifdef BREADBOARD
	out     ddrb,r17
#endif

	ldi     r17,0x60
	or      r17,r16
	out     portc,r17       
	                        ; 125 ns/clock @ 8 MHz)
	ldi     r17,0x40        ; 1 clock (need 92 nsec)
	or      r17,r16         ; 1 clock
	out     portc,r17       ; 1 clock: activate ior-

	isa_delay               ; need 552 nsec
	ldi     r17,0x60        ; ~ior/~iow inactive
	or      r17,r16         ; keep the address bits constant

	in      i0,pina
	in      i1,pinb

	out     portc,r17
	;;;isa_delay            ;; BDL!!!

	pop     r17
	ret

.global isa_reset
isa_reset:                      ; isa bus reset
;;
;; set I/O port C as output, and A/B as inputs.  A/B change direction when
;; we do ISA transactions, but C is always an output.
;;
	ser r16                     ; r16 = 0xff
	out     ddrc,r16                ; c is an output - ALWAYS!
	clr r16
	out ddra,r16                ; leave A/B as inputs for now
#ifdef BREADBOARD
	out ddrb,r16
#endif
;;
;; RESET the ISA BUS - ending with making ~ior and ~iow INACTIVE.
;;
	ldi     r16,0xe0                 ; drop ~ior and ~iow on ISA bus, raise reset
	out portc,r16
	rcall delay
	ldi r16,0x60
	out portc,r16
	rcall delay
	ret

delay:                    ; delay ~64K clocks (~25 msecs @ 8 MHz)
	push r16
	push r17
	clr r16
delay1:
	clr r17
delayw:
	dec r17            ; +1 clock * 64K
	brne    delayw     ; +2 clocks * 64K
	dec r16            ; +1 clock * 256
	brne    delay1     : +2 clocks * 256
	pop r17
	pop r16
	ret

